# Copyright 2018 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""Example CLI command for running upgrade checks"""

import sys

from oslo_config import cfg
from oslo_upgradecheck import upgradecheck


class Checks(upgradecheck.UpgradeCommands):
    def success(self):
        return upgradecheck.Result(upgradecheck.Code.SUCCESS,
                                   'Always succeeds')

    def failure(self):
        return upgradecheck.Result(upgradecheck.Code.FAILURE, 'Always fails')

    _upgrade_checks = (('always succeeds', success),
                       ('always fails', failure),
                       )


def main():
    return upgradecheck.main(
        conf=cfg.CONF,
        project='myprojectname',
        upgrade_command=Checks(),
    )


if __name__ == '__main__':
    sys.exit(main())



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""
test_upgradecheck
----------------------------------

Tests for `upgradecheck` module.
"""

import os.path
import subprocess
import sys
from unittest import mock

from oslo_config import cfg
from oslotest import base

from oslo_upgradecheck import upgradecheck


class TestUpgradeCheckResult(base.BaseTestCase):

    def test_details(self):
        result = upgradecheck.Result(upgradecheck.Code.SUCCESS, 'test details')
        self.assertEqual(0, result.code)
        self.assertEqual('test details', result.details)


class TestCommands(upgradecheck.UpgradeCommands):
    def success(self):
        return upgradecheck.Result(upgradecheck.Code.SUCCESS,
                                   'Always succeeds')

    def warning(self):
        return upgradecheck.Result(upgradecheck.Code.WARNING, 'Always warns')

    def failure(self):
        return upgradecheck.Result(upgradecheck.Code.FAILURE, 'Always fails')

    _upgrade_checks = (('always succeeds', success),
                       ('always warns', warning),
                       ('always fails', failure),
                       )


class SuccessCommands(TestCommands):
    _upgrade_checks = ()


class TestUpgradeCommands(base.BaseTestCase):
    def test_get_details(self):
        result = upgradecheck.Result(upgradecheck.Code.SUCCESS, '*' * 70)
        upgrade_commands = upgradecheck.UpgradeCommands()
        details = upgrade_commands._get_details(result)
        wrapped = '*' * 60 + '\n  ' + '*' * 10
        self.assertEqual(wrapped, details)

    def test_check(self):
        inst = TestCommands()
        result = inst.check()
        self.assertEqual(upgradecheck.Code.FAILURE, result)


class TestMain(base.BaseTestCase):
    def _run_test(self, upgrade_command, expected):
        conf = cfg.ConfigOpts()
        result = upgradecheck.main(
            conf=conf,
            project='oslo-upgradecheck-test',
            upgrade_command=upgrade_command,
            argv=['upgrade', 'check'],
        )
        self.assertEqual(expected, result)

    def test_main(self):
        inst = TestCommands()
        self._run_test(inst, upgradecheck.Code.FAILURE)

    def test_main_exception(self):
        raises = mock.Mock()
        raises.check.side_effect = Exception('test exception')
        self._run_test(raises, 255)

    def test_main_success(self):
        inst = SuccessCommands()
        self._run_test(inst, 0)


class TestExampleFile(base.BaseTestCase):
    def test_example_main(self):
        path = os.path.join(os.path.dirname(os.path.abspath(__file__)),
                            '../../doc/source/main.py')
        # The example includes both a passing and failing test, which means the
        # overall result is failure.
        self.assertEqual(
            upgradecheck.Code.FAILURE,
            subprocess.call([sys.executable, path, 'upgrade', 'check']))



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import fixtures
import os.path
import tempfile
import yaml

from oslo_config import cfg
from oslo_config import fixture as config
from oslo_policy import opts as policy_opts
from oslo_serialization import jsonutils
from oslotest import base

from oslo_upgradecheck import common_checks
from oslo_upgradecheck import upgradecheck


class TestUpgradeCheckPolicyJSON(base.BaseTestCase):

    def setUp(self):
        super().setUp()
        conf_fixture = self.useFixture(config.Config())
        conf_fixture.load_raw_values()
        self.conf = conf_fixture.conf
        self.conf.register_opts(policy_opts._options,
                                group=policy_opts._option_group)

        self.cmd = upgradecheck.UpgradeCommands()
        self.cmd._upgrade_checks = (('Policy File JSON to YAML Migration',
                                    (common_checks.check_policy_json,
                                     {'conf': self.conf})),)

        self.data = {
            'rule_admin': 'True',
            'rule_admin2': 'is_admin:True'
        }
        self.temp_dir = self.useFixture(fixtures.TempDir())
        fd, self.json_file = tempfile.mkstemp(dir=self.temp_dir.path)
        fd, self.yaml_file = tempfile.mkstemp(dir=self.temp_dir.path)
        with open(self.json_file, 'w') as fh:
            jsonutils.dump(self.data, fh)
        with open(self.yaml_file, 'w') as fh:
            yaml.dump(self.data, fh)

        original_search_dirs = cfg._search_dirs

        def fake_search_dirs(dirs, name):
            dirs.append(self.temp_dir.path)
            return original_search_dirs(dirs, name)

        mock_search_dir = self.useFixture(
            fixtures.MockPatch('oslo_config.cfg._search_dirs')).mock
        mock_search_dir.side_effect = fake_search_dirs

    def test_policy_json_file_fail_upgrade(self):
        # Test with policy json file full path set in config.
        self.conf.set_override('policy_file', self.json_file,
                               group="oslo_policy")
        self.assertEqual(upgradecheck.Code.FAILURE,
                         self.cmd.check())

    def test_policy_yaml_file_pass_upgrade(self):
        # Test with full policy yaml file path set in config.
        self.conf.set_override('policy_file', self.yaml_file,
                               group="oslo_policy")
        self.assertEqual(upgradecheck.Code.SUCCESS,
                         self.cmd.check())

    def test_no_policy_file_pass_upgrade(self):
        # Test with no policy file exist, means use policy from code.
        self.conf.set_override('policy_file', 'non_exist_file',
                               group="oslo_policy")
        self.assertEqual(upgradecheck.Code.SUCCESS,
                         self.cmd.check())

    def test_default_policy_yaml_file_pass_upgrade(self):
        self.conf.set_override('policy_file', 'policy.yaml',
                               group="oslo_policy")
        tmpfilename = os.path.join(self.temp_dir.path, 'policy.yaml')
        with open(tmpfilename, 'w') as fh:
            yaml.dump(self.data, fh)
        self.assertEqual(upgradecheck.Code.SUCCESS,
                         self.cmd.check())

    def test_old_default_policy_json_file_fail_upgrade(self):
        self.conf.set_override('policy_file', 'policy.json',
                               group="oslo_policy")
        tmpfilename = os.path.join(self.temp_dir.path, 'policy.json')
        with open(tmpfilename, 'w') as fh:
            jsonutils.dump(self.data, fh)
        self.assertEqual(upgradecheck.Code.FAILURE,
                         self.cmd.check())



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_policy import opts as policy_opts
from oslo_utils import fileutils

from oslo_upgradecheck import upgradecheck

"""
Common checks which can be used by multiple services.
"""


def check_policy_json(self, conf):
    "Checks to see if policy file is JSON-formatted policy file."

    # NOTE(gmann): This method need [oslo_policy].policy_file
    # config value so register those options in case they
    # are not register by services.
    conf.register_opts(policy_opts._options,
                       group=policy_opts._option_group)

    msg = ("Your policy file is JSON-formatted which is "
           "deprecated. You need to switch to YAML-formatted file. "
           "Use the ``oslopolicy-convert-json-to-yaml`` "
           "tool to convert the existing JSON-formatted files to "
           "YAML in a backwards-compatible manner: "
           "https://docs.openstack.org/oslo.policy/"
           "latest/cli/oslopolicy-convert-json-to-yaml.html.")
    status = upgradecheck.Result(upgradecheck.Code.SUCCESS)
    # Check if policy file exist and is JSON-formatted.
    policy_path = conf.find_file(conf.oslo_policy.policy_file)
    if policy_path and fileutils.is_json(policy_path):
        status = upgradecheck.Result(upgradecheck.Code.FAILURE, msg)
    return status
